-- SID: 1.6
-- Date Modified: 01/09/18 17:43:14
--
-- _DefineGenTrapDescription.sql
--
-- Module Description
--   Defines the _DefineGenTrapDescription stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- _DefineGenTrapDescription
PRINT 'Creating procedure _DefineGenTrapDescription'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DefineGenTrapDescription' AND type = 'P')
  DROP PROCEDURE _DefineGenTrapDescription
GO

-- SID: 1.6
-- Date Modified: 01/09/18 17:43:14
-- Creates or updates a GenericTrapDescription.
CREATE PROCEDURE _DefineGenTrapDescription
  @cid          ClassID,
  @id           ObjID,
  @ParentCID    NVARCHAR(4),         -- Parent class id of GenericTrapCategory
  @ParentID     INT,                 -- Parent instance id of GenericTrapCategory. 0 indicates a class-owned category.
  @Category     NVARCHAR(54),        -- Category Name. Must be unique for a given ParentCID/ParentID.
  @MsgID        NVARCHAR(255),       -- Message id of trap to register. Can contain '*' to indicate wildcard.
  @Enabled      TINYINT = NULL,      -- 0=Disabled, 1=Enabled. Optional parameter to specify whether trap is enabled for the specified parent category.
  @Description  NVARCHAR(255) = NULL,-- If specified, then the description of the trap is updated.
  @SelectRow    TINYINT = NULL,      -- If not specified, or set to 1 then the new/updated GenericTrapDescription fields are selected
  @ObjectId     INT = NULL OUTPUT    -- Optional output parameter to receive id of object created/updated.
AS
  /*
  PRINT '_DefineGenTrapDescription'
  PRINT '  @cid          = ' + CASE WHEN @cid IS NULL THEN 'NULL' ELSE '''' + @cid END + ''', '
  PRINT '  @id           = ' + CASE WHEN @id IS NULL THEN 'NULL' ELSE CAST(@id AS NVARCHAR) END + ', '
  PRINT '  @ParentCID    = ' + CASE WHEN @ParentCID IS NULL THEN 'NULL' ELSE '''' + @ParentCID END + ''', '
  PRINT '  @ParentID     = ' + CASE WHEN @ParentID IS NULL THEN 'NULL' ELSE CAST(@ParentID AS NVARCHAR) END + ', '
  PRINT '  @Category     = ' + CASE WHEN @Category IS NULL THEN 'NULL' ELSE '''' + @Category END + ''', '
  PRINT '  @MsgID        = ' + CASE WHEN @MsgID IS NULL THEN 'NULL' ELSE '''' + @MsgID END + ''', '
  PRINT '  @Enabled      = ' + CASE WHEN @Enabled IS NULL THEN 'NULL' ELSE CAST(@Enabled AS NVARCHAR) END + ', '
  PRINT '  @Description  = ' + CASE WHEN @Description IS NULL THEN 'NULL' ELSE '''' END + @Description + ''''
  */

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION _DefineGenTrapDescription
  ELSE
    SAVE TRANSACTION _DefineGenTrapDescription
     
  DECLARE @gtde_id ObjID, @gtct_id ObjID, @gtde_deleted TINYINT, @gtct_deleted TINYINT
  SELECT @gtde_id = gtde.id, @gtde_deleted = gtde.deleted, @gtct_id = gtct.id, @gtct_deleted = gtct.deleted
  FROM GenericTrapCategory_V AS gtct
    LEFT OUTER JOIN GenericTrapDescription_V AS gtde ON gtct.id = gtde._ParentID AND _MsgID = @MsgID
  WHERE gtct._ParentCID = @ParentCID AND gtct._ParentID = @ParentID AND gtct.name = @Category

  -- Undelete objects marked as deleted  
  IF @gtct_deleted = 1
    UPDATE GenericTrapCategory_V SET deleted = 0 WHERE id = @gtct_id
  IF @gtde_deleted = 1
    UPDATE GenericTrapDescription_V SET deleted = 0 WHERE id = @gtde_id

  -- See if we have to create the category
  IF @gtct_id IS NULL
  BEGIN
    EXEC @gtct_id = asisp_createGTCT
      @name = @Category,
      @parent_cid = @ParentCID,
      @parent_id = @ParentID,
      @Desc = @Category
    IF @gtct_id IS NULL OR @gtct_id < 0
    BEGIN
      RAISERROR('_DefineGenTrapDescription: Could not define new GenericTrapCategory @ParentCID=''%s'', @ParentID=%d, @Category=''%s''', 3, 1,
                @ParentCID, @ParentID, @Category)
      ROLLBACK TRANSACTION _DefineGenTrapDescription
      IF @gtct_id IS NULL
        SET @gtct_id = -30000
    END
  END

  IF @gtde_id IS NULL
  BEGIN
    IF @Enabled IS NULL
      SET @Enabled = 1
    IF @Description IS NULL
      SET @Description = 1
    EXEC @gtde_id = asisp_createGTDE
      @name = @MsgID,
      @parent_cid = 'GTCT',
      @parent_id = @gtct_id,
      @MsgID  = @MsgID,
      @Enabled = @Enabled,
      @Desc = @Description
    IF @gtde_id IS NULL OR @gtde_id < 0
    BEGIN
      RAISERROR('_DefineGenTrapDescription: Could not define new GenericTrapDescription @ParentCID=''%s'', @ParentID=%d, @Category=''%s'', @MsgID=''%s''', 3, 1,
                @ParentCID, @ParentID, @Category, @MsgID)
      ROLLBACK TRANSACTION _DefineGenTrapDescription
      IF @gtde_id IS NULL
        SET @gtde_id = -30000
      RETURN @gtde_id
    END
  END
  ELSE
  BEGIN
    UPDATE GenericTrapDescription_V SET
      _Enabled = ISNULL(@Enabled, _Enabled),
      _Desc    = ISNULL(@Description, _Desc)
    WHERE
      id = @gtde_id
      AND
      (
        (@Enabled IS NOT NULL AND _Enabled <> @Enabled)
        OR
        (@Description IS NOT NULL AND _Desc <> @Description)
      )
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('_DefineGenTrapDescription: Could not update GenericTrapDescription(%d) @ParentCID=''%s'', @ParentID=%s, @Category=''%s'', @MsgID=''%s''', 3, 1,
                @gtde_id, @ParentCID, @ParentID, @Category, @MsgID)
      ROLLBACK TRANSACTION _DefineGenTrapDescription
      RETURN -30000
    END
  END

  -- Select the new/updated row
  IF @SelectRow IS NULL OR @SelectRow <> 0
    SELECT * FROM GenericTrapDescription_V WHERE id = @gtde_id

  IF @tcount = 0
    COMMIT TRANSACTION _DefineGenTrapDescription

  SET @ObjectId = @gtde_id
  RETURN @gtde_id
GO
